#!/usr/bin/perl

use strict;

if (@ARGV < 3)
{
    print "Usage: sr.pl old_string new_string file(s)\n";
    exit;
}

my $old = shift @ARGV;
my $new = shift @ARGV;

my $filename;

foreach $filename (@ARGV)
{
    print "Processing $filename ...\n";

    # If the file does not exist, move along.
    unless (-e $filename)
    {
        print "Skipping $filename, it does not exist.\n";
        next;
    }

    # If the file is a directory, move along.
    if (-d $filename)
    {
        print "Skipping $filename, it's a directory.\n";
        next;
    }


    # If the file is not readable, move along.
    unless (-r $filename)
    {
        print "Skipping $filename, it's not readable.\n";
        next;
    }

    # If the file is not writable, move along.
    unless (-w $filename)
    {
        print "Skipping $filename, it's not writable.\n";
        next;
    }

    # If the file is not a text file, move along.
    if (-B $filename)
    {
        print "Skipping $filename, it's a binary file.\n";
        next;
    }

    my $backup_filename = $filename . "~";

    # If the backup file already exists, raise an error
    if (-e $backup_filename)
    {
        print "Skipping $filename, backup file $backup_filename exists.\n";
        next;
    }

    rename $filename, $backup_filename;

    open (IN, "< $backup_filename")
        or die "Can't open $backup_filename";

    open (OUT, "> $filename")
        or die "Can't write to $filename";

    while (<IN>)
    {
        s/$old/$new/g;
        print OUT;
    }

    close IN;
    close OUT;
}

