#!/usr/local/bin/perl

use strict;
use CGI;
use CGI::Carp qw(fatalsToBrowser);
use XML::Simple;
use LWP::Simple;

my $portfolios = XMLin("./portfolio.xml", forcearray => 1);
my $command = "list";
my $query = new CGI;
my $quote_url = 'http://finance.yahoo.com/q?d=v1&s=';
my $stock_info_url = 'http://finance.yahoo.com/q?d=t&s=';
my $errors = "";

if ($query->param('c'))
{
    $command = $query->param('c');
}

print $query->header;
print "<html><head><title>portfolio</title></head><body>\n";
print "<h1 align=\"center\">Stock Portfolio</h1>\n";

if ($command eq "list")
{
    &list_portfolios;
    &display_add_form;
}
elsif ($command eq "view")
{
    &view_portfolio($query->param('p'));
    &display_add_stock_form($query->param('p'));
}
elsif ($command eq "del")
{
    &delete_portfolio($query->param('p'));
    &list_portfolios;
    &display_add_form;
}
elsif ($command eq "del_stock")
{
    &delete_stock($query->param('p'), $query->param('s'));
    &view_portfolio($query->param('p'));
    &display_add_stock_form($query->param('p'));
}
elsif ($command eq "add")
{
    if ($query->param('p'))
    {
        &add_portfolio($query->param('p'));
    }
    else
    {
        $errors .= "<li>You must enter a portfolio name.</li>\n";
    }

    &list_portfolios;
    &display_add_form;
}
elsif ($command eq "add_stock")
{
    if ($query->param('s'))
    {
        &add_stock($query->param('p'), $query->param('s'));
    }
    else
    {
        $errors .= "<li>You must enter a ticker symbol.</li>\n";
    }

    &view_portfolio($query->param('p'));
    &display_add_stock_form($query->param('p'));
}

# Write out the update data file.
&write_data;
print "<p><a href=\"portfolio.pl\">return to portfolio list</a></p>\n";
print "</body></html>\n";

sub list_portfolios
{
    my $hashref = $portfolios->{portfolio};

    print "<div align=\"center\">\n";
    print "<table cellpadding=\"8\">\n";
    foreach my $port (keys %$hashref)
    {
        my $encoded_port = &encode_string($port);
        my $view_link = "portfolio.pl?c=view&amp;p=$encoded_port";
        my $delete_link = "portfolio.pl?c=del&amp;p=$encoded_port";
        print "<tr>\n";
        print "<td><b>$port</b></td>\n";
        print "<td>";
        print "<a href=\"$view_link\">view</a></td>\n";
        print "<td><a href=\"$delete_link\">delete</a></td>\n";
        print "</tr>\n";
    }
    print "</table>\n";
    print "</div>\n";
}

sub view_portfolio
{
    my $port = shift(@_);

    my $hashref = $portfolios->{portfolio}->{$port}->{stock};

    print "<h3>$port</h3>";
    foreach my $stock_ticker (keys %$hashref)
    {
        my $stock = $hashref->{$stock_ticker};
        my $company_name = $stock->{company};
        my $delete_link = "portfolio.pl?c=del_stock&amp;p=$port";
        $delete_link .= "&s=" . $stock_ticker;

        print "<p>\n";
        print "<table border=\"1\" cellspacing=\"0\" cellpadding=\"5\">\n";
        print "<tr><td colspan=\"2\"><b>", $company_name, "</b>"; 
        print "<br /><font size=\"1\">";
        print "<a href=\"$delete_link\">delete</a>";
        print "</font>";
        print "</td></tr>\n";

        my $current_price = &get_current_price($stock_ticker); 

        my $lastprice = $stock->{lastprice};
        my $lastfetch = $stock->{lastfetch};

        # Move this soon.
        $stock->{lastprice} = $current_price;
        $stock->{lastfetch} = time;

        print "<tr><td>ticker</td>";
        print "<td>", $stock_ticker, "</td></tr>\n";
        print "<tr><td>current price:</td>";
        print "<td>", $current_price, "</td></tr>\n";
        print "<tr><td>last price:</td>";
        print "<td>", $lastprice, "</td></tr>\n";
        print "<tr><td>as of:</td>";
        print "<td>", scalar localtime($lastfetch), "</td></tr>\n";
        print "</table>\n";
        print "</p>\n";
    }
}

sub delete_portfolio
{
    my $portfolio_name = shift(@_);

    delete $portfolios->{portfolio}->{$portfolio_name};
}

sub delete_stock
{
    my $portfolio_name = shift(@_);
    my $ticker = shift(@_);

    delete $portfolios->{portfolio}->{$portfolio_name}->{stock}->{$ticker};
}

sub get_current_price
{
    my $ticker = shift(@_);
    my $url = $quote_url . "$ticker";
    my $content = get($url);

    if (!$content)
    {
        return "request failed";
    }

    my $current_price = "not found";

    if ($content =~ /<td[^>]*><font[^>]*><a[^>]*>(\w+?)<\/a><\/font><\/td><td[^>]*><font[^>]*>(.+?)<\/font><\/td><td[^>]*><font[^>]*><b>([\d.]+?)<\/b><\/font><\/td>/i)
    {
        $current_price = $3;
    }

    return $current_price;
}

sub add_portfolio
{
    my $portfolio_name = shift(@_);

    if (exists $portfolios->{portfolio}->{$portfolio_name})
    {
        $errors .= "<li>That portfolio name is already in use.</li>\n";
        return;
    }

    $portfolios->{portfolio}->{$portfolio_name} = {};
}

sub add_stock
{
    my $portfolio = shift(@_);
    my $ticker = shift(@_);
    my $stock_name = "";
    my $stock_price = 0;

    if (exists $portfolios->{portfolio}->{$portfolio}->{stock}->{$ticker})
    {
        $errors .= "<li>That stock is already in the portfolio.</li>\n";
    }

    my $url = $stock_info_url . "$ticker";
    my $content = get($url);

    if (!$content)
    {
        $errors .= "<li>Couldn't retrieve stock information.</li>\n";
        return;
    }

    if ($content =~ /No such ticker symbol/i)
    {
        $errors .= "<li>Invalid ticker symbol.</li>\n";
        return;
    }

    if ($content =~ /<td colspan=7><font(.+?)><b>(.+?)\s+<\/b>/i)
    {
        $stock_name = $2;
    }

    if ($content =~ /Last Trade<br>(.+?)<b>([\d.]+)<\/b>/i)
    {
        $stock_price = $2;
    }

    my $hashref = $portfolios->{portfolio}->{$portfolio}->{stock};

    $hashref->{$ticker} = { lastfetch => time, lastprice => $stock_price, 
        company => $stock_name };
}

sub write_data
{
    my $xml = XMLout($portfolios);
    open (FILE, "> portfolio.xml")
        or die "Can't open data file: ";
    print FILE $xml;
    close FILE;
}

sub display_add_form
{
    print "<h3 align=\"center\">add a new portfolio</h3>\n";
    if ($errors)
    {
        print "<div align=\"center\"><table><tr><td>\n";
        print "Please correct the following error:\n";
        print "<ul>\n", $errors, "</ul>\n";
        print "</td></tr></table>\n";
    }
    print "<form>\n";
    print "<input type=\"hidden\" name=\"c\" value=\"add\">\n";
    print "<div align=\"center\"><table>\n";
    print "<tr><td>portfolio name:</td><td>";
    print "<input type=\"text\" name=\"p\" value=\"\">";
    print "</td></tr>\n";
    print "<tr><td colspan=\"2\"><input type=\"submit\"></td></tr>\n";
    print "</table></div>\n";
    print "</form>\n";
}

sub display_add_stock_form
{
    my $portfolio_name = shift(@_);
    print "<h3 align=\"center\">add a new stock</h3>\n";
    if ($errors)
    {
        print "<div align=\"center\"><table><tr><td>\n";
        print "Please correct the following error:\n";
        print "<ul>\n", $errors, "</ul>\n";
        print "</td></tr></table>\n";
    }
    print "<form>\n";
    print "<input type=\"hidden\" name=\"c\" value=\"add_stock\">\n";
    print "<input type=\"hidden\" name=\"p\" value=\"$portfolio_name\">\n";
    print "<div align=\"center\"><table>\n";
    print "<tr><td>stock ticker symbol:</td><td>";
    print "<input type=\"text\" name=\"s\" value=\"\">";
    print "</td></tr>\n";
    print "<tr><td colspan=\"2\"><input type=\"submit\"></td></tr>\n";
    print "</table></div>\n";
    print "</form>\n";
}

sub encode_string
{
    my $string_to_encode = shift @_;

    $string_to_encode =~ s/\&/%26/g;
    $string_to_encode =~ s/\+/%2B/g;
    $string_to_encode =~ s/\?/%2B/g;
    $string_to_encode =~ s/ /+/g;

    return $string_to_encode;
}
